import dataclasses
from . import DEFAULT_LANG as DEFAULT_LANG, ID3_V2 as ID3_V2, ID3_V2_2 as ID3_V2_2, ID3_V2_3 as ID3_V2_3, ID3_V2_4 as ID3_V2_4, LATIN1_ENCODING as LATIN1_ENCODING, UTF_16BE_ENCODING as UTF_16BE_ENCODING, UTF_16_ENCODING as UTF_16_ENCODING, UTF_8_ENCODING as UTF_8_ENCODING, apple as apple
from .. import Error as Error, core as core
from ..utils.binfuncs import bin2bytes as bin2bytes, bin2dec as bin2dec, bytes2bin as bytes2bin, bytes2dec as bytes2dec, bytes2signedInt16 as bytes2signedInt16, dec2bin as dec2bin, dec2bytes as dec2bytes, signedInt162bytes as signedInt162bytes
from .headers import ExtendedTagHeader, FrameHeader as FrameHeader, TagHeader
from _typeshed import Incomplete
from typing import NamedTuple

ISO_8859_1: str

class FrameException(Error): ...

TITLE_FID: bytes
SUBTITLE_FID: bytes
ARTIST_FID: bytes
ALBUM_ARTIST_FID: bytes
ORIG_ARTIST_FID: bytes
COMPOSER_FID: bytes
ALBUM_FID: bytes
TRACKNUM_FID: bytes
GENRE_FID: bytes
COMMENT_FID: bytes
USERTEXT_FID: bytes
OBJECT_FID: bytes
UNIQUE_FILE_ID_FID: bytes
LYRICS_FID: bytes
DISCNUM_FID: bytes
IMAGE_FID: bytes
USERURL_FID: bytes
PLAYCOUNT_FID: bytes
BPM_FID: bytes
PUBLISHER_FID: bytes
CDID_FID: bytes
PRIVATE_FID: bytes
TOS_FID: bytes
POPULARITY_FID: bytes
ENCODED_BY_FID: bytes
COPYRIGHT_FID: bytes
URL_COMMERCIAL_FID: bytes
URL_COPYRIGHT_FID: bytes
URL_AUDIOFILE_FID: bytes
URL_ARTIST_FID: bytes
URL_AUDIOSRC_FID: bytes
URL_INET_RADIO_FID: bytes
URL_PAYMENT_FID: bytes
URL_PUBLISHER_FID: bytes
URL_FIDS: list[bytes]
TOC_FID: bytes
CHAPTER_FID: bytes
DEPRECATED_DATE_FIDS: list[bytes]
DATE_FIDS: list[bytes]

class Frame:
    id: bytes
    decompressed_size: int
    group_id: bytes
    encrypt_method: Incomplete
    data: Incomplete
    data_len: int
    def __init__(self, id: bytes) -> None: ...
    @property
    def header(self) -> Incomplete: ...
    @header.setter
    def header(self, h: Incomplete) -> None: ...
    def parse(self, data: bytes, frame_header: Incomplete) -> None: ...
    def render(self) -> bytes: ...
    def __lt__(self, other: object) -> bool: ...
    @staticmethod
    def decompress(data: bytes) -> bytes: ...
    @staticmethod
    def compress(data: bytes) -> bytes: ...
    @staticmethod
    def decrypt(data: bytes) -> bytes: ...
    @staticmethod
    def encrypt(data: bytes) -> bytes: ...
    @property
    def text_delim(self) -> bytes: ...
    @property
    def encoding(self) -> bytes: ...
    @encoding.setter
    def encoding(self, enc: bytes) -> None: ...
    @property
    def strict_rendering(self) -> bool: ...
    @property
    def unknown(self) -> bool: ...

class TextFrame(Frame):
    def __init__(self, id: bytes, text: Incomplete | None = None) -> None: ...
    @property
    def text(self) -> str: ...
    @text.setter
    def text(self, txt: str) -> None: ...
    encoding: Incomplete
    def parse(self, data: Incomplete, frame_header: Incomplete) -> None: ...
    data: Incomplete
    def render(self) -> Incomplete: ...

class UserTextFrame(TextFrame):
    def __init__(self, id: bytes =..., description: str = '', text: str = '') -> None: ...
    @property
    def description(self) -> str: ...
    @description.setter
    def description(self, txt: str) -> None: ...
    encoding: Incomplete
    text: Incomplete
    def parse(self, data: Incomplete, frame_header: Incomplete) -> None: ...
    data: Incomplete
    def render(self) -> Incomplete: ...

class DateFrame(TextFrame):
    encoding: Incomplete
    def __init__(self, id: bytes, date: str = '') -> None: ...
    def parse(self, data: Incomplete, frame_header: Incomplete) -> None: ...
    @property
    def date(self) -> core.Date: ...
    @date.setter
    def date(self, date: core.Date) -> None: ...

class UrlFrame(Frame):
    encoding: Incomplete
    def __init__(self, id: bytes, url: str = '') -> None: ...
    @property
    def url(self) -> str | None: ...
    @url.setter
    def url(self, url: str | None) -> None: ...
    def parse(self, data: Incomplete, frame_header: Incomplete) -> None: ...
    data: Incomplete
    def render(self) -> Incomplete: ...

class UserUrlFrame(UrlFrame):
    def __init__(self, id: bytes =..., description: str = '', url: str = '') -> None: ...
    @property
    def description(self) -> str: ...
    @description.setter
    def description(self, desc: str) -> None: ...
    encoding: bytes
    url: Incomplete
    def parse(self, data: Incomplete, frame_header: Incomplete) -> None: ...
    data: Incomplete
    def render(self) -> Incomplete: ...

class UnknownFrame(Frame):
    def __init__(self, id: bytes) -> None: ...

class ImageFrame(Frame):
    OTHER: int
    ICON: int
    OTHER_ICON: int
    FRONT_COVER: int
    BACK_COVER: int
    LEAFLET: int
    MEDIA: int
    LEAD_ARTIST: int
    ARTIST: int
    CONDUCTOR: int
    BAND: int
    COMPOSER: int
    LYRICIST: int
    RECORDING_LOCATION: int
    DURING_RECORDING: int
    DURING_PERFORMANCE: int
    VIDEO: int
    BRIGHT_COLORED_FISH: int
    ILLUSTRATION: int
    BAND_LOGO: int
    PUBLISHER_LOGO: int
    MIN_TYPE = OTHER
    MAX_TYPE = PUBLISHER_LOGO
    URL_MIME_TYPE: bytes
    URL_MIME_TYPE_STR: str
    URL_MIME_TYPE_VALUES: Incomplete
    image_data: bytes
    image_url: str | None
    def __init__(self, id: str =..., description: str = '', image_data: Incomplete | None = None, image_url: Incomplete | None = None, picture_type: Incomplete | None = None, mime_type: Incomplete | None = None) -> None: ...
    @property
    def description(self) -> str: ...
    @description.setter
    def description(self, d: str) -> None: ...
    @property
    def mime_type(self) -> str: ...
    @mime_type.setter
    def mime_type(self, m: bytes | str) -> None: ...
    @property
    def picture_type(self) -> int | None: ...
    @picture_type.setter
    def picture_type(self, t: int | None) -> None: ...
    encoding: Incomplete
    desciption: str
    def parse(self, data: Incomplete, frame_header: Incomplete) -> None: ...
    data: Incomplete
    def render(self) -> Incomplete: ...
    @staticmethod
    def picTypeToString(t: int) -> str: ...
    @staticmethod
    def stringToPicType(s: str) -> int: ...
    def makeFileName(self, name: str | None = None) -> str: ...

class ObjectFrame(Frame):
    object_data: Incomplete
    def __init__(self, fid: Incomplete =..., description: str = '', filename: str = '', object_data: Incomplete | None = None, mime_type: Incomplete | None = None) -> None: ...
    @property
    def description(self) -> str: ...
    @description.setter
    def description(self, txt: str) -> None: ...
    @property
    def mime_type(self) -> str: ...
    @mime_type.setter
    def mime_type(self, m: bytes | str) -> None: ...
    @property
    def filename(self) -> str: ...
    @filename.setter
    def filename(self, txt: str) -> None: ...
    encoding: Incomplete
    def parse(self, data: Incomplete, frame_header: Incomplete) -> None: ...
    data: Incomplete
    def render(self) -> Incomplete: ...

class PrivateFrame(Frame):
    owner_id: bytes
    owner_data: bytes
    def __init__(self, id: bytes =..., owner_id: bytes = b'', owner_data: bytes = b'') -> None: ...
    def parse(self, data: Incomplete, frame_header: Incomplete) -> None: ...
    data: Incomplete
    def render(self) -> Incomplete: ...

class MusicCDIdFrame(Frame):
    def __init__(self, id: bytes =..., toc: bytes = b'') -> None: ...
    @property
    def toc(self) -> Incomplete: ...
    data: Incomplete
    @toc.setter
    def toc(self, toc: Incomplete) -> None: ...
    def parse(self, data: Incomplete, frame_header: Incomplete) -> None: ...

class PlayCountFrame(Frame):
    count: Incomplete
    def __init__(self, id: bytes =..., count: int = 0) -> None: ...
    def parse(self, data: Incomplete, frame_header: Incomplete) -> None: ...
    data: Incomplete
    def render(self) -> Incomplete: ...

class PopularityFrame(Frame):
    def __init__(self, id: bytes =..., email: bytes = b'', rating: int = 0, count: int = 0) -> None: ...
    @property
    def rating(self) -> int: ...
    @rating.setter
    def rating(self, rating: int) -> None: ...
    @property
    def email(self) -> str: ...
    @email.setter
    def email(self, email: str | bytes) -> None: ...
    @property
    def count(self) -> int: ...
    @count.setter
    def count(self, count: int) -> None: ...
    def parse(self, data: Incomplete, frame_header: Incomplete) -> None: ...
    data: Incomplete
    def render(self) -> Incomplete: ...

class UniqueFileIDFrame(Frame):
    def __init__(self, id: bytes =..., owner_id: bytes = b'', uniq_id: bytes = b'') -> None: ...
    @property
    def owner_id(self) -> bytes: ...
    @owner_id.setter
    def owner_id(self, oid: bytes) -> None: ...
    @property
    def uniq_id(self) -> bytes: ...
    @uniq_id.setter
    def uniq_id(self, uid: bytes) -> None: ...
    def parse(self, data: Incomplete, frame_header: Incomplete) -> None: ...
    data: Incomplete
    def render(self) -> Incomplete: ...

class LanguageCodeMixin:
    @property
    def lang(self) -> bytes: ...
    @lang.setter
    def lang(self, lang: bytes) -> None: ...

class DescriptionLangTextFrame(Frame, LanguageCodeMixin):
    def __init__(self, id: bytes, description: str, lang: bytes, text: str) -> None: ...
    @property
    def description(self) -> str: ...
    @description.setter
    def description(self, description: str) -> None: ...
    @property
    def text(self) -> str: ...
    @text.setter
    def text(self, text: str) -> None: ...
    encoding: Incomplete
    def parse(self, data: Incomplete, frame_header: Incomplete) -> None: ...
    data: Incomplete
    def render(self) -> Incomplete: ...

class CommentFrame(DescriptionLangTextFrame):
    def __init__(self, id: bytes =..., description: str = '', lang: bytes = ..., text: str = '') -> None: ...

class LyricsFrame(DescriptionLangTextFrame):
    def __init__(self, id: bytes =..., description: str = '', lang: bytes =..., text: str = '') -> None: ...

class TermsOfUseFrame(Frame, LanguageCodeMixin):
    lang: Incomplete
    def __init__(self, id: bytes = b'USER', text: str = '', lang: bytes =...) -> None: ...
    @property
    def text(self) -> str: ...
    @text.setter
    def text(self, text: str) -> None: ...
    encoding: Incomplete
    def parse(self, data: Incomplete, frame_header: Incomplete) -> None: ...
    data: Incomplete
    def render(self) -> Incomplete: ...

class TocFrame(Frame):
    TOP_LEVEL_FLAG_BIT: int
    ORDERED_FLAG_BIT: int
    element_id: bytes
    toplevel: Incomplete
    ordered: Incomplete
    child_ids: Incomplete
    description: Incomplete
    def __init__(self, id: bytes =..., element_id: bytes | None = None, toplevel: bool = True, ordered: bool = True, child_ids: Incomplete | None = None, description: Incomplete | None = None) -> None: ...
    def parse(self, data: Incomplete, frame_header: Incomplete) -> None: ...
    data: Incomplete
    def render(self) -> Incomplete: ...

class RelVolAdjFrameV24(Frame):
    CHANNEL_TYPE_OTHER: int
    CHANNEL_TYPE_MASTER: int
    CHANNEL_TYPE_FRONT_RIGHT: int
    CHANNEL_TYPE_FRONT_LEFT: int
    CHANNEL_TYPE_BACK_RIGHT: int
    CHANNEL_TYPE_BACK_LEFT: int
    CHANNEL_TYPE_FRONT_CENTER: int
    CHANNEL_TYPE_BACK_CENTER: int
    CHANNEL_TYPE_BASS: int
    @property
    def identifier(self) -> str: ...
    @identifier.setter
    def identifier(self, ident: str | bytes) -> None: ...
    @property
    def channel_type(self) -> int: ...
    @channel_type.setter
    def channel_type(self, t: int) -> None: ...
    @property
    def adjustment(self) -> int: ...
    @adjustment.setter
    def adjustment(self, adj: int) -> None: ...
    @property
    def peak(self) -> Incomplete: ...
    @peak.setter
    def peak(self, v: Incomplete) -> None: ...
    def __init__(self, fid: bytes = b'RVA2', identifier: Incomplete | None = None, channel_type: Incomplete | None = None, adjustment: Incomplete | None = None, peak: Incomplete | None = None) -> None: ...
    def parse(self, data: Incomplete, frame_header: Incomplete) -> None: ...
    header: Incomplete
    data: Incomplete
    def render(self) -> Incomplete: ...

class RelVolAdjFrameV23(Frame):
    FRONT_CHANNEL_RIGHT_BIT: int
    FRONT_CHANNEL_LEFT_BIT: int
    BACK_CHANNEL_RIGHT_BIT: int
    BACK_CHANNEL_LEFT_BIT: int
    FRONT_CENTER_CHANNEL_BIT: int
    BASS_CHANNEL_BIT: int
    CHANNEL_DEFN: Incomplete
    @dataclasses.dataclass
    class VolumeAdjustments:
        master: int = ...
        master_peak: int = ...
        front_right: int = ...
        front_left: int = ...
        front_right_peak: int = ...
        front_left_peak: int = ...
        back_right: int = ...
        back_left: int = ...
        back_right_peak: int = ...
        back_left_peak: int = ...
        front_center: int = ...
        front_center_peak: int = ...
        back_center: int = ...
        back_center_peak: int = ...
        bass: int = ...
        bass_peak: int = ...
        other: int = ...
        other_peak: int = ...
        @property
        def has_master_channel(self) -> bool: ...
        @property
        def has_front_channel(self) -> bool: ...
        @property
        def has_back_channel(self) -> bool: ...
        @property
        def has_front_center_channel(self) -> bool: ...
        @property
        def has_back_center_channel(self) -> bool: ...
        @property
        def has_bass_channel(self) -> bool: ...
        @property
        def has_other_channel(self) -> bool: ...
        def boundsCheck(self) -> None: ...
        def setChannelAdj(self, chan_type: Incomplete, value: Incomplete) -> None: ...
        def setChannelPeak(self, chan_type: Incomplete, value: Incomplete) -> None: ...
    adjustments: Incomplete
    def __init__(self, fid: bytes = b'RVAD') -> None: ...
    def toV24(self) -> list[RelVolAdjFrameV24]: ...
    def parse(self, data: Incomplete, frame_header: Incomplete) -> None: ...
    header: Incomplete
    data: Incomplete
    def render(self) -> Incomplete: ...

class StartEndTuple(NamedTuple):
    start: Incomplete
    end: Incomplete

class ChapterFrame(Frame):
    NO_OFFSET: int
    element_id: bytes
    times: Incomplete
    offsets: Incomplete
    sub_frames: Incomplete
    def __init__(self, id: bytes=..., element_id: bytes | None = None, times: Incomplete | None = None, offsets: Incomplete | None = None, sub_frames: Incomplete | None = None) -> None: ...
    def parse(self, data: Incomplete, frame_header: Incomplete) -> None: ...
    data: Incomplete
    def render(self) -> Incomplete: ...
    @property
    def title(self) -> str | None: ...
    @title.setter
    def title(self, title: str | None) -> None: ...
    @property
    def subtitle(self) -> str | None: ...
    @subtitle.setter
    def subtitle(self, subtitle: str | None) -> None: ...
    @property
    def user_url(self) -> str | None: ...
    @user_url.setter
    def user_url(self, url: str | None) -> None: ...

class FrameSet(dict[Incomplete, list[Frame]]):
    def __init__(self) -> None: ...
    def parse(self, f: Incomplete, tag_header: TagHeader, extended_header: ExtendedTagHeader) -> int: ...
    def __getitem__(self, fid: Incomplete) -> list[Frame] | None: ... # type: ignore
    def __setitem__(self, fid: Incomplete, frame: Frame) -> None: ... # type: ignore
    @property
    def unknown_frame_ids(self) -> set[Incomplete]: ...
    def getAllFrames(self) -> list[Frame]: ...
    def setTextFrame(self, fid: Incomplete, text: str) -> None: ...
    def __contains__(self, fid: Incomplete) -> bool: ...

ID3_FRAMES: Incomplete

TAGS2_2_TO_TAGS_2_3_AND_4: Incomplete
NONSTANDARD_ID3_FRAMES: Incomplete
